// Animation.java
//
// Informatics 45 Spring 2010
// Code Example: Ball Animation (Version 1)
//
// This class implements the logic behind our animation.  It keeps track of
// the current position and radius of the ball in terms of fractional
// coordinates, updating them according to a "frame rate" (i.e., the number
// of frames per second).
//
// The animation itself runs in a separate thread, so that it can update the
// position of the ball periodically while allowing the GUI to remain
// responsive (so, for example, the "Stop" button can be pressed at any time).

package inf45.spring2010.examples.animation1.model;

import java.util.ArrayList;


public class Animation
{
	private Thread animationThread;
	private double framesPerSecond;
	
	private ArrayList<AnimationListener> listeners;
	
	
	public Animation(double framesPerSecond)
	{
		animationThread = null;
		this.framesPerSecond = framesPerSecond;
		
		listeners = new ArrayList<AnimationListener>();
	}
	
	
	public void addAnimationListener(AnimationListener listener)
	{
		listeners.add(listener);
	}
	
	
	public void removeAnimationListener(AnimationListener listener)
	{
		if (listeners.contains(listener))
		{
			listeners.remove(listener);
		}
	}

	
	public void start()
	{
		animationThread = new Thread(
			new Runnable()
			{
				public void run()
				{
					runAnimation();
				}
			});
		
		animationThread.start();
	}
	
	
	public void stop()
	{
		if (animationThread != null)
		{
			animationThread.interrupt();
			
			try
			{
				animationThread.join();
			}
			catch (InterruptedException e)
			{
			}
			
			animationThread = null;
		}
	}
	
	
	private void runAnimation()
	{
		try
		{
			fireAnimationStarted();
			
			// These variables track the current position and radius of
			// the ball as the animation progresses.
			double ballCenterX = 0.2;
			double ballCenterY = 0.3;
			double ballRadius = 0.05;
			
			while (true)
			{
				// Here, we sleep for a number of milliseconds determined
				// by the number of frames per second.  (For example, if
				// there are to be 40 frames per second, we sleep for
				// 1000/40 = 25 milliseconds.
				Thread.sleep((int) (1000 / framesPerSecond));

				// Next, we calculate the new position of the ball.  Note
				// the use of framesPerSecond in the formula, which will
				// allow the speed of the ball to be calculated independently
				// of the number of frames per second (so, for example, we
				// can increase the number of frames per second without
				// affecting the speed of the ball).
				ballCenterX += 0.4 / framesPerSecond;
				ballCenterY += 0.2 / framesPerSecond;

				// Finally, now that we've updated the ball position, we
				// tell the GUI that the state of the animation has changed
				// and can be redrawn.
				fireAnimationUpdated(
					new AnimationState(ballCenterX, ballCenterY, ballRadius));
			}
		}
		catch (InterruptedException e)
		{
			// Interruption is used to get out of this thread, as it was in
			// the "word printing" code example from earlier this quarter.
		}
		
		fireAnimationStopped();
	}
	
	
	private void fireAnimationStarted()
	{
		for (AnimationListener listener : listeners)
		{
			listener.animationStarted();
		}
	}
	
	
	private void fireAnimationUpdated(AnimationState newState)
	{
		for (AnimationListener listener : listeners)
		{
			listener.animationUpdated(newState);
		}
	}
	
	
	private void fireAnimationStopped()
	{
		for (AnimationListener listener : listeners)
		{
			listener.animationStopped();
		}
	}
}
